#! /usr/bin/env python
# ----------------------------------
#
# prot_discretizer.py
#
# Usage: prot_optimizer.py
#
# Read test protocols and round run times upwards towards
# discrete values selectable  via command line switches:
#
# --constant: Round all times to 300 (arbitray)
# --equidist[=<val>]: Round all values to the nearest integral
#   multiple of <val>. If <val> is omitted, assumes 10.
# --prop: Round all times to 1,2,5,10,20,50,100,...

import sys
import re
import string
import math

ed_opt = "--equidist"
ed_optspc = ed_opt+"="
ed_spclen = len(ed_optspc);


def const_round(val):
        return 300
    
class equidist_round:
    def __init__(self, distance):
        self.distance = distance
        
    def __call__(self, val):
        return int(math.ceil(float(val)/self.distance)*self.distance)

def prop_round(val):
    base = 1;
    while val > 5:
        base = base*10;
        val = val/10;
    if val < 1:
        res = 1
    elif val < 2:
        res = 2
    else:
        res = 5
    return res*base



class process_line:
    def __init__(self, round_fun):
        self.round_fun = round_fun;

    def __call__(self,line):
        if line[0] == "#":
           return line
        parts = string.split(line, None, 4)
        return "%-29s %s %8.3f %010s %s" % (parts[0], parts[1],
                                            self.round_fun(float(parts[2])),
                                            parts[3],parts[4])

# ----- End of Definitions ------------

round_fun = equidist_round(10)

options = filter(lambda x:x[0:2]=="--", sys.argv) # Take out options
files   = filter(lambda x:x[0:2]!="--", sys.argv)[1:] # Take out options ;-)

for opt in options:
    if opt == "--constant":
        round_fun = const_round
    elif opt == "--prop":
        round_fun = prop_round
    elif opt == ed_opt:
        round_fun = equidist_round(10)
    elif opt[0:ed_spclen] == ed_optspc:
        try:
            round_fun = equidist_round(int(opt[ed_spclen:]))
        except ValueError:
            sys.stderr.write("Wrong argument to option "+equidist+".\n")
            raise
    else:
        sys.stderr.write("Unknown option "+opt+".\n")
        raise "OptionError"
    

for file in files:
    f=open(file, "r")
    lines = f.readlines()
    res = map(process_line(round_fun), lines);
    for line in res:
        print line
        
